# $Id: /mirror/coderepos/lang/perl/Data-Feed/trunk/lib/Data/Feed/Feed.pm 102544 2009-03-19T08:58:09.853141Z daisuke  $

package Data::Feed::Feed;
use Any::Moose '::Role';

no Any::Moose '::Role';

1;

__END__

=head1 NAME

Data::Feed::Feed - Role For A Feed 

=cut